@@ -17,6 +17,7 @@
 import datetime
 import sys
 import tempfile
+
 from collections import OrderedDict
 from decimal import Decimal
 from random import random
@@ -33,13 +34,11 @@ from bson import (Decimal128,
                   _BUILT_IN_TYPES,
                   _dict_to_bson,
                   _bson_to_dict)
-from bson.code import Code
 from bson.codec_options import (CodecOptions, TypeCodec, TypeDecoder,
                                 TypeEncoder, TypeRegistry)
 from bson.errors import InvalidDocument
 from bson.int64 import Int64
 from bson.raw_bson import RawBSONDocument
-from bson.py3compat import text_type
 
 from gridfs import GridIn, GridOut
 
@@ -49,7 +48,7 @@ from pymongo.message import _CursorAddress
 
 from test import client_context, unittest
 from test.test_client import IntegrationTest
-from test.utils import ignore_deprecations, rs_client
+from test.utils import rs_client
 
 
 class DecimalEncoder(TypeEncoder):
@@ -110,7 +109,7 @@ UNINT_CODECOPTS = CodecOptions(type_registry=TypeRegistry(
 
 
 class UppercaseTextDecoder(TypeDecoder):
-    bson_type = text_type
+    bson_type = str
     def transform_bson(self, value):
         return value.upper()
 
@@ -588,19 +587,6 @@ class TestCollectionWCustomType(IntegrationTest):
         for doc_cls in [RawBSONDocument, OrderedDict]:
             run_test(doc_cls)
 
-    @client_context.require_version_max(4, 1, 0, -1)
-    def test_group_w_custom_type(self):
-        db = self.db
-        test = db.get_collection('test', codec_options=UNINT_CODECOPTS)
-        test.insert_many([
-            {'sku': 'a', 'qty': UndecipherableInt64Type(2)},
-            {'sku': 'b', 'qty': UndecipherableInt64Type(5)},
-            {'sku': 'a', 'qty': UndecipherableInt64Type(1)}])
-
-        self.assertEqual([{'sku': 'b', 'qty': UndecipherableInt64Type(5)},],
-                         test.group(["sku", "qty"], {"sku": "b"}, {},
-                                    "function (obj, prev) { }"))
-
     def test_aggregate_w_custom_type_decoder(self):
         db = self.db
         db.test.insert_many([
@@ -635,39 +621,6 @@ class TestCollectionWCustomType(IntegrationTest):
 
         self.assertEqual(values, test.distinct("a"))
 
-    def test_map_reduce_w_custom_type(self):
-        test = self.db.get_collection(
-            'test', codec_options=UPPERSTR_DECODER_CODECOPTS)
-
-        test.insert_many([
-            {'_id': 1, 'sku': 'abcd', 'qty': 1},
-            {'_id': 2, 'sku': 'abcd', 'qty': 2},
-            {'_id': 3, 'sku': 'abcd', 'qty': 3}])
-
-        map = Code("function () {"
-                   "  emit(this.sku, this.qty);"
-                   "}")
-        reduce = Code("function (key, values) {"
-                      "  return Array.sum(values);"
-                      "}")
-
-        result = test.map_reduce(map, reduce, out={'inline': 1})
-        self.assertTrue(isinstance(result, dict))
-        self.assertTrue('results' in result)
-        self.assertEqual(result['results'][0], {'_id': 'ABCD', 'value': 6})
-
-        result = test.inline_map_reduce(map, reduce)
-        self.assertTrue(isinstance(result, list))
-        self.assertEqual(1, len(result))
-        self.assertEqual(result[0]["_id"], 'ABCD')
-
-        full_result = test.inline_map_reduce(map, reduce,
-                                             full_response=True)
-        result = full_result['results']
-        self.assertTrue(isinstance(result, list))
-        self.assertEqual(1, len(result))
-        self.assertEqual(result[0]["_id"], 'ABCD')
-
     def test_find_one_and__w_custom_type_decoder(self):
         db = self.db
         c = db.get_collection('test', codec_options=UNINT_DECODER_CODECOPTS)
@@ -692,22 +645,6 @@ class TestCollectionWCustomType(IntegrationTest):
         self.assertEqual(doc['x'].value, 3)
         self.assertIsNone(c.find_one())
 
-    @ignore_deprecations
-    def test_find_and_modify_w_custom_type_decoder(self):
-        db = self.db
-        c = db.get_collection('test', codec_options=UNINT_DECODER_CODECOPTS)
-        c.insert_one({'_id': 1, 'x': Int64(1)})
-
-        doc = c.find_and_modify({'_id': 1}, {'$inc': {'x': Int64(10)}})
-        self.assertEqual(doc['_id'], 1)
-        self.assertIsInstance(doc['x'], UndecipherableInt64Type)
-        self.assertEqual(doc['x'].value, 1)
-
-        doc = c.find_one()
-        self.assertEqual(doc['_id'], 1)
-        self.assertIsInstance(doc['x'], UndecipherableInt64Type)
-        self.assertEqual(doc['x'].value, 11)
-
 
 class TestGridFileCustomType(IntegrationTest):
     def setUp(self):
@@ -735,7 +672,7 @@ class TestGridFileCustomType(IntegrationTest):
         self.assertEqual(["foo"], two.aliases)
         self.assertEqual({"foo": 'red', "bar": 'blue'}, two.metadata)
         self.assertEqual(3, two.bar)
-        self.assertEqual("5eb63bbbe01eeed093cb22bb8f5acdc3", two.md5)
+        self.assertEqual(None, two.md5)
 
         for attr in ["_id", "name", "content_type", "length", "chunk_size",
                      "upload_date", "aliases", "metadata", "md5"]:
@@ -853,11 +790,11 @@ class ChangeStreamsWCustomTypesTestMixin(object):
 class TestCollectionChangeStreamsWCustomTypes(
     IntegrationTest, ChangeStreamsWCustomTypesTestMixin):
     @classmethod
-    @client_context.require_version_min(3, 6, 0)
     @client_context.require_no_mmap
     @client_context.require_no_standalone
     def setUpClass(cls):
         super(TestCollectionChangeStreamsWCustomTypes, cls).setUpClass()
+        cls.db.test.delete_many({})
 
     def tearDown(self):
         self.input_target.drop()
@@ -879,6 +816,7 @@ class TestDatabaseChangeStreamsWCustomTypes(
     @client_context.require_no_standalone
     def setUpClass(cls):
         super(TestDatabaseChangeStreamsWCustomTypes, cls).setUpClass()
+        cls.db.test.delete_many({})
 
     def tearDown(self):
         self.input_target.drop()
@@ -900,6 +838,7 @@ class TestClusterChangeStreamsWCustomTypes(
     @client_context.require_no_standalone
     def setUpClass(cls):
         super(TestClusterChangeStreamsWCustomTypes, cls).setUpClass()
+        cls.db.test.delete_many({})
 
     def tearDown(self):
         self.input_target.drop()
