@@ -22,7 +22,6 @@ sys.path[0:0] = [""]
 
 from bson.int64 import Int64
 from bson.objectid import ObjectId
-from bson.py3compat import text_type
 from bson.son import SON
 from pymongo import CursorType, monitoring, InsertOne, UpdateOne, DeleteOne
 from pymongo.command_cursor import CommandCursor
@@ -33,22 +32,22 @@ from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   client_knobs,
-                  PyMongoTestCase,
+                  IntegrationTest,
                   sanitize_cmd,
                   unittest)
 from test.utils import (EventListener,
                         get_pool,
-                        ignore_deprecations,
                         rs_or_single_client,
                         single_client,
                         wait_until)
 
 
-class TestCommandMonitoring(PyMongoTestCase):
+class TestCommandMonitoring(IntegrationTest):
 
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
+        super(TestCommandMonitoring, cls).setUpClass()
         cls.listener = EventListener()
         cls.client = rs_or_single_client(
             event_listeners=[cls.listener],
@@ -57,9 +56,11 @@ class TestCommandMonitoring(PyMongoTestCase):
     @classmethod
     def tearDownClass(cls):
         cls.client.close()
+        super(TestCommandMonitoring, cls).tearDownClass()
 
     def tearDown(self):
         self.listener.results.clear()
+        super(TestCommandMonitoring, self).tearDown()
 
     def test_started_simple(self):
         self.client.pymongo_test.command('ping')
@@ -814,231 +815,7 @@ class TestCommandMonitoring(PyMongoTestCase):
         error = errors[0]
         self.assertEqual(0, error.get('index'))
         self.assertIsInstance(error.get('code'), int)
-        self.assertIsInstance(error.get('errmsg'), text_type)
-
-    def test_legacy_writes(self):
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
-
-            coll = self.client.pymongo_test.test
-            coll.drop()
-            self.listener.results.clear()
-
-            # Implied write concern insert
-            _id = coll.insert({'x': 1})
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('insert', coll.name),
-                            ('ordered', True),
-                            ('documents', [{'_id': _id, 'x': 1}])])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('insert', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(1, reply.get('n'))
-
-            # Unacknowledged insert
-            self.listener.results.clear()
-            _id = coll.insert({'x': 1}, w=0)
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('insert', coll.name),
-                            ('ordered', True),
-                            ('documents', [{'_id': _id, 'x': 1}]),
-                            ('writeConcern', {'w': 0})])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('insert', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            self.assertEqual(succeeded.reply, {'ok': 1})
-
-            # Explicit write concern insert
-            self.listener.results.clear()
-            _id = coll.insert({'x': 1}, w=1)
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('insert', coll.name),
-                            ('ordered', True),
-                            ('documents', [{'_id': _id, 'x': 1}]),
-                            ('writeConcern', {'w': 1})])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('insert', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(1, reply.get('n'))
-
-            # remove all
-            self.listener.results.clear()
-            res = coll.remove({'x': 1}, w=1)
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('delete', coll.name),
-                            ('ordered', True),
-                            ('deletes', [SON([('q', {'x': 1}),
-                                              ('limit', 0)])]),
-                            ('writeConcern', {'w': 1})])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('delete', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(res['n'], reply.get('n'))
-
-            # upsert
-            self.listener.results.clear()
-            oid = ObjectId()
-            coll.update({'_id': oid}, {'_id': oid, 'x': 1}, upsert=True, w=1)
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('update', coll.name),
-                            ('ordered', True),
-                            ('updates', [SON([('q', {'_id': oid}),
-                                              ('u', {'_id': oid, 'x': 1}),
-                                              ('multi', False),
-                                              ('upsert', True)])]),
-                            ('writeConcern', {'w': 1})])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('update', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(1, reply.get('n'))
-            self.assertEqual([{'index': 0, '_id': oid}], reply.get('upserted'))
-
-            # update one
-            self.listener.results.clear()
-            coll.update({'x': 1}, {'$inc': {'x': 1}})
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('update', coll.name),
-                            ('ordered', True),
-                            ('updates', [SON([('q', {'x': 1}),
-                                              ('u', {'$inc': {'x': 1}}),
-                                              ('multi', False),
-                                              ('upsert', False)])])])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('update', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(1, reply.get('n'))
-
-            # update many
-            self.listener.results.clear()
-            coll.update({'x': 2}, {'$inc': {'x': 1}}, multi=True)
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('update', coll.name),
-                            ('ordered', True),
-                            ('updates', [SON([('q', {'x': 2}),
-                                              ('u', {'$inc': {'x': 1}}),
-                                              ('multi', True),
-                                              ('upsert', False)])])])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('update', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(1, reply.get('n'))
-
-            # remove one
-            self.listener.results.clear()
-            coll.remove({'x': 3}, multi=False)
-            results = self.listener.results
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = SON([('delete', coll.name),
-                            ('ordered', True),
-                            ('deletes', [SON([('q', {'x': 3}),
-                                              ('limit', 1)])])])
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('pymongo_test', started.database_name)
-            self.assertEqual('delete', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            reply = succeeded.reply
-            self.assertEqual(1, reply.get('ok'))
-            self.assertEqual(1, reply.get('n'))
-
-            self.assertEqual(0, coll.count_documents({}))
+        self.assertIsInstance(error.get('errmsg'), str)
 
     def test_insert_many(self):
         # This always uses the bulk API.
@@ -1082,51 +859,47 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertEqual(documents, docs)
         self.assertEqual(6, count)
 
-    def test_legacy_insert_many(self):
-        # On legacy servers this uses bulk OP_INSERT.
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", DeprecationWarning)
+    def test_insert_many_unacknowledged(self):
+        coll = self.client.pymongo_test.test
+        coll.drop()
+        unack_coll = coll.with_options(write_concern=WriteConcern(w=0))
+        self.listener.results.clear()
 
-            coll = self.client.pymongo_test.test
-            coll.drop()
-            self.listener.results.clear()
-
-            # Force two batches on legacy servers.
-            big = 'x' * (1024 * 1024 * 12)
-            docs = [{'_id': i, 'big': big} for i in range(6)]
-            coll.insert(docs)
-            results = self.listener.results
-            started = results['started']
-            succeeded = results['succeeded']
-            self.assertEqual(0, len(results['failed']))
-            documents = []
-            count = 0
-            operation_id = started[0].operation_id
-            self.assertIsInstance(operation_id, int)
-            for start, succeed in zip(started, succeeded):
-                self.assertIsInstance(start, monitoring.CommandStartedEvent)
-                cmd = sanitize_cmd(start.command)
-                self.assertEqual(['insert', 'ordered', 'documents'],
-                                 list(cmd.keys()))
-                self.assertEqual(coll.name, cmd['insert'])
-                self.assertIs(True, cmd['ordered'])
-                documents.extend(cmd['documents'])
-                self.assertEqual('pymongo_test', start.database_name)
-                self.assertEqual('insert', start.command_name)
-                self.assertIsInstance(start.request_id, int)
-                self.assertEqual(self.client.address, start.connection_id)
-                self.assertIsInstance(succeed, monitoring.CommandSucceededEvent)
-                self.assertIsInstance(succeed.duration_micros, int)
-                self.assertEqual(start.command_name, succeed.command_name)
-                self.assertEqual(start.request_id, succeed.request_id)
-                self.assertEqual(start.connection_id, succeed.connection_id)
-                self.assertEqual(start.operation_id, operation_id)
-                self.assertEqual(succeed.operation_id, operation_id)
-                reply = succeed.reply
-                self.assertEqual(1, reply.get('ok'))
-                count += reply.get('n', 0)
-            self.assertEqual(documents, docs)
-            self.assertEqual(6, count)
+        # Force two batches on legacy servers.
+        big = 'x' * (1024 * 1024 * 12)
+        docs = [{'_id': i, 'big': big} for i in range(6)]
+        unack_coll.insert_many(docs)
+        results = self.listener.results
+        started = results['started']
+        succeeded = results['succeeded']
+        self.assertEqual(0, len(results['failed']))
+        documents = []
+        operation_id = started[0].operation_id
+        self.assertIsInstance(operation_id, int)
+        for start, succeed in zip(started, succeeded):
+            self.assertIsInstance(start, monitoring.CommandStartedEvent)
+            cmd = sanitize_cmd(start.command)
+            cmd.pop('writeConcern', None)
+            self.assertEqual(['insert', 'ordered', 'documents'],
+                             list(cmd.keys()))
+            self.assertEqual(coll.name, cmd['insert'])
+            self.assertIs(True, cmd['ordered'])
+            documents.extend(cmd['documents'])
+            self.assertEqual('pymongo_test', start.database_name)
+            self.assertEqual('insert', start.command_name)
+            self.assertIsInstance(start.request_id, int)
+            self.assertEqual(self.client.address, start.connection_id)
+            self.assertIsInstance(succeed, monitoring.CommandSucceededEvent)
+            self.assertIsInstance(succeed.duration_micros, int)
+            self.assertEqual(start.command_name, succeed.command_name)
+            self.assertEqual(start.request_id, succeed.request_id)
+            self.assertEqual(start.connection_id, succeed.connection_id)
+            self.assertEqual(start.operation_id, operation_id)
+            self.assertEqual(succeed.operation_id, operation_id)
+            self.assertEqual(1, succeed.reply.get('ok'))
+        self.assertEqual(documents, docs)
+        wait_until(lambda: coll.count_documents({}) == 6,
+                   'insert documents with w=0')
 
     def test_bulk_write(self):
         coll = self.client.pymongo_test.test
@@ -1208,7 +981,7 @@ class TestCommandMonitoring(PyMongoTestCase):
             'data': {
                 'failCommands': ['insert'],
                 'closeConnection': False,
-                'errorCode': 10107,  # NotPrimary
+                'errorCode': 10107,  # Not primary
             },
         }
         with self.fail_point(insert_command_error):
@@ -1222,29 +995,6 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertEqual(event.failure['code'], 10107)
         self.assertTrue(event.failure['errmsg'])
 
-    @client_context.require_version_max(3, 4, 99)
-    def test_bulk_write_legacy_network_error(self):
-        self.listener.results.clear()
-
-        # Make the delete operation run on a closed connection.
-        self.client.admin.command('ping')
-        pool = get_pool(self.client)
-        sock_info = pool.sockets[0]
-        sock_info.sock.close()
-
-        # Test legacy unacknowledged write network error.
-        coll = self.client.pymongo_test.get_collection(
-            'test', write_concern=WriteConcern(w=0))
-        with self.assertRaises(AutoReconnect):
-            coll.bulk_write([InsertOne({'_id': 1})], ordered=False)
-        failed = self.listener.results['failed']
-        self.assertEqual(1, len(failed))
-        event = failed[0]
-        self.assertEqual(event.command_name, 'insert')
-        self.assertIsInstance(event.failure, dict)
-        self.assertEqual(event.failure['errtype'], 'AutoReconnect')
-        self.assertTrue(event.failure['errmsg'])
-
     def test_write_errors(self):
         coll = self.client.pymongo_test.test
         coll.drop()
@@ -1310,50 +1060,6 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertTrue('ok' in succeeded.reply)
 
         self.listener.results.clear()
-        self.client.pymongo_test.current_op(True)
-        started = results['started'][0]
-        succeeded = results['succeeded'][0]
-        self.assertEqual(0, len(results['failed']))
-        self.assertIsInstance(started, monitoring.CommandStartedEvent)
-        expected = SON([('currentOp', 1), ('$all', True)])
-        self.assertEqualCommand(expected, started.command)
-        self.assertEqual('admin', started.database_name)
-        self.assertEqual('currentOp', started.command_name)
-        self.assertIsInstance(started.request_id, int)
-        self.assertEqual(self.client.address, started.connection_id)
-        self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-        self.assertIsInstance(succeeded.duration_micros, int)
-        self.assertEqual(started.command_name, succeeded.command_name)
-        self.assertEqual(started.request_id, succeeded.request_id)
-        self.assertEqual(started.connection_id, succeeded.connection_id)
-        self.assertTrue('inprog' in succeeded.reply)
-        self.assertTrue('ok' in succeeded.reply)
-
-        if not client_context.is_mongos:
-            with ignore_deprecations():
-                self.client.fsync(lock=True)
-                self.listener.results.clear()
-                self.client.unlock()
-                # Wait for async unlock...
-                wait_until(
-                    lambda: not self.client.is_locked, "unlock the database")
-            started = results['started'][0]
-            succeeded = results['succeeded'][0]
-            self.assertEqual(0, len(results['failed']))
-            self.assertIsInstance(started, monitoring.CommandStartedEvent)
-            expected = {'fsyncUnlock': 1}
-            self.assertEqualCommand(expected, started.command)
-            self.assertEqual('admin', started.database_name)
-            self.assertEqual('fsyncUnlock', started.command_name)
-            self.assertIsInstance(started.request_id, int)
-            self.assertEqual(self.client.address, started.connection_id)
-            self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-            self.assertIsInstance(succeeded.duration_micros, int)
-            self.assertEqual(started.command_name, succeeded.command_name)
-            self.assertEqual(started.request_id, succeeded.request_id)
-            self.assertEqual(started.connection_id, succeeded.connection_id)
-            self.assertTrue('info' in succeeded.reply)
-            self.assertTrue('ok' in succeeded.reply)
 
     def test_sensitive_commands(self):
         listeners = self.client._event_listeners
@@ -1384,11 +1090,12 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertEqual({}, succeeded.reply)
 
 
-class TestGlobalListener(PyMongoTestCase):
+class TestGlobalListener(IntegrationTest):
 
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
+        super(TestGlobalListener, cls).setUpClass()
         cls.listener = EventListener()
         # We plan to call register(), which internally modifies _LISTENERS.
         cls.saved_listeners = copy.deepcopy(monitoring._LISTENERS)
@@ -1401,8 +1108,10 @@ class TestGlobalListener(PyMongoTestCase):
     def tearDownClass(cls):
         monitoring._LISTENERS = cls.saved_listeners
         cls.client.close()
+        super(TestGlobalListener, cls).tearDownClass()
 
     def setUp(self):
+        super(TestGlobalListener, self).setUp()
         self.listener.results.clear()
 
     def test_simple(self):
@@ -1422,7 +1131,7 @@ class TestGlobalListener(PyMongoTestCase):
         self.assertTrue(isinstance(started.request_id, int))
 
 
-class TestEventClasses(PyMongoTestCase):
+class TestEventClasses(unittest.TestCase):
 
     def test_command_event_repr(self):
         request_id, connection_id, operation_id = 1, ('localhost', 27017), 2
