@@ -16,19 +16,21 @@
 
 https://github.com/mongodb/specifications/blob/master/source/unified-test-format/unified-test-format.rst
 """
-
+import collections
 import copy
 import datetime
 import functools
 import os
 import re
 import sys
+import time
 import types
 
+from collections import abc
+
 from bson import json_util, Code, Decimal128, DBRef, SON, Int64, MaxKey, MinKey
 from bson.binary import Binary
 from bson.objectid import ObjectId
-from bson.py3compat import abc, integer_types, iteritems, text_type, PY3
 from bson.regex import Regex, RE_TYPE
 
 from gridfs import GridFSBucket
@@ -44,7 +46,7 @@ from pymongo.errors import (
 from pymongo.monitoring import (
     CommandFailedEvent, CommandListener, CommandStartedEvent,
     CommandSucceededEvent, _SENSITIVE_COMMANDS, PoolCreatedEvent,
-    PoolClearedEvent, PoolClosedEvent, ConnectionCreatedEvent,
+    PoolReadyEvent, PoolClearedEvent, PoolClosedEvent, ConnectionCreatedEvent,
     ConnectionReadyEvent, ConnectionClosedEvent,
     ConnectionCheckOutStartedEvent, ConnectionCheckOutFailedEvent,
     ConnectionCheckedOutEvent, ConnectionCheckedInEvent)
@@ -67,6 +69,13 @@ from test.utils import (
 
 JSON_OPTS = json_util.JSONOptions(tz_aware=False)
 
+IS_INTERRUPTED = False
+
+
+def interrupt_loop():
+    global IS_INTERRUPTED
+    IS_INTERRUPTED = True
+
 
 def with_metaclass(meta, *bases):
     """Create a base class with a metaclass.
@@ -174,9 +183,6 @@ class NonLazyCursor(object):
         # Create the server side cursor.
         self.first_result = next(find_cursor, None)
 
-    def __iter__(self):
-        return self
-
     def __next__(self):
         if self.first_result is not None:
             first = self.first_result
@@ -184,15 +190,13 @@ class NonLazyCursor(object):
             return first
         return next(self.find_cursor)
 
-    next = __next__
-
     def close(self):
         self.find_cursor.close()
 
 
 class EventListenerUtil(CMAPListener, CommandListener):
     def __init__(self, observe_events, ignore_commands,
-                 observe_sensitive_commands):
+                 observe_sensitive_commands, store_events, entity_map):
         self._event_types = set(name.lower() for name in observe_events)
         if observe_sensitive_commands:
             self._observe_sensitive_commands = True
@@ -201,6 +205,15 @@ class EventListenerUtil(CMAPListener, CommandListener):
             self._observe_sensitive_commands = False
             self._ignore_commands = _SENSITIVE_COMMANDS | set(ignore_commands)
             self._ignore_commands.add('configurefailpoint')
+        self._event_mapping = collections.defaultdict(list)
+        self.entity_map = entity_map
+        if store_events:
+            for i in store_events:
+                id = i["id"]
+                events = (i.lower() for i in i["events"])
+                for i in events:
+                    self._event_mapping[i].append(id)
+                self.entity_map[id] = []
         super(EventListenerUtil, self).__init__()
 
     def get_events(self, event_type):
@@ -209,8 +222,15 @@ class EventListenerUtil(CMAPListener, CommandListener):
         return [e for e in self.events if 'Command' not in type(e).__name__]
 
     def add_event(self, event):
-        if type(event).__name__.lower() in self._event_types:
+        event_name = type(event).__name__.lower()
+        if event_name in self._event_types:
             super(EventListenerUtil, self).add_event(event)
+        for id in self._event_mapping[event_name]:
+            self.entity_map[id].append({
+                "name": type(event).__name__,
+                "observedAt": time.time(),
+                "description": repr(event)
+            })
 
     def _command_event(self, event):
         if event.command_name.lower() not in self._ignore_commands:
@@ -245,6 +265,12 @@ class EntityMapUtil(object):
         self._session_lsids = {}
         self.test = test_class
 
+    def __contains__(self, item):
+        return item in self._entities
+
+    def __len__(self):
+        return len(self._entities)
+
     def __getitem__(self, item):
         try:
             return self._entities[item]
@@ -253,7 +279,7 @@ class EntityMapUtil(object):
                 item,))
 
     def __setitem__(self, key, value):
-        if not isinstance(key, text_type):
+        if not isinstance(key, str):
             self.test.fail(
                 'Expected entity name of type str, got %s' % (type(key)))
 
@@ -262,26 +288,26 @@ class EntityMapUtil(object):
 
         self._entities[key] = value
 
-    def _create_entity(self, entity_spec):
+    def _create_entity(self, entity_spec, uri=None):
         if len(entity_spec) != 1:
             self.test.fail(
                 "Entity spec %s did not contain exactly one top-level key" % (
                     entity_spec,))
 
-        entity_type, spec = next(iteritems(entity_spec))
+        entity_type, spec = next(iter(entity_spec.items()))
         if entity_type == 'client':
             kwargs = {}
             observe_events = spec.get('observeEvents', [])
             ignore_commands = spec.get('ignoreCommandMonitoringEvents', [])
             observe_sensitive_commands = spec.get(
                 'observeSensitiveCommands', False)
-            # TODO: PYTHON-2511 support storeEventsAsEntities
-            if len(observe_events) or len(ignore_commands):
-                ignore_commands = [cmd.lower() for cmd in ignore_commands]
-                listener = EventListenerUtil(
-                    observe_events, ignore_commands, observe_sensitive_commands)
-                self._listeners[spec['id']] = listener
-                kwargs['event_listeners'] = [listener]
+            ignore_commands = [cmd.lower() for cmd in ignore_commands]
+            listener = EventListenerUtil(
+                observe_events, ignore_commands,
+                observe_sensitive_commands,
+                spec.get("storeEventsAsEntities"), self)
+            self._listeners[spec['id']] = listener
+            kwargs['event_listeners'] = [listener]
             if spec.get('useMultipleMongoses'):
                 if client_context.load_balancer or client_context.serverless:
                     kwargs['h'] = client_context.MULTI_MONGOS_LB_URI
@@ -293,6 +319,8 @@ class EntityMapUtil(object):
                 kwargs['server_api'] = ServerApi(
                     server_api['version'], strict=server_api.get('strict'),
                     deprecation_errors=server_api.get('deprecationErrors'))
+            if uri:
+                kwargs['h'] = uri
             client = rs_or_single_client(**kwargs)
             self[spec['id']] = client
             self.test.addCleanup(client.close)
@@ -344,9 +372,9 @@ class EntityMapUtil(object):
         self.test.fail(
             'Unable to create entity of unknown type %s' % (entity_type,))
 
-    def create_entities_from_spec(self, entity_spec):
+    def create_entities_from_spec(self, entity_spec, uri=None):
         for spec in entity_spec:
-            self._create_entity(spec)
+            self._create_entity(spec, uri=uri)
 
     def get_listener_for_client(self, client_name):
         client = self[client_name]
@@ -376,21 +404,16 @@ class EntityMapUtil(object):
             return self._session_lsids[session_name]
 
 
-if not PY3:
-    binary_types = (Binary,)
-    long_types = (Int64, long)
-    unicode_type = unicode
-else:
-    binary_types = (Binary, bytes)
-    long_types = (Int64,)
-    unicode_type = str
+binary_types = (Binary, bytes)
+long_types = (Int64,)
+unicode_type = str
 
 
 BSON_TYPE_ALIAS_MAP = {
     # https://docs.mongodb.com/manual/reference/operator/query/type/
     # https://pymongo.readthedocs.io/en/stable/api/bson/index.html
     'double': (float,),
-    'string': (text_type,),
+    'string': (str,),
     'object': (abc.Mapping,),
     'array': (abc.MutableSequence,),
     'binData': binary_types,
@@ -405,7 +428,7 @@ BSON_TYPE_ALIAS_MAP = {
     'symbol': (unicode_type,),
     'javascriptWithScope': (unicode_type, Code),
     'int': (int,),
-    'long': long_types,
+    'long': (Int64,),
     'decimal': (Decimal128,),
     'maxKey': (MaxKey,),
     'minKey': (MinKey,),
@@ -499,11 +522,11 @@ class MatchEvaluatorUtil(object):
             else:
                 nested = expectation[key_to_compare]
                 if isinstance(nested, abc.Mapping) and len(nested) == 1:
-                    opname, spec = next(iteritems(nested))
+                    opname, spec = next(iter(nested.items()))
                     if opname.startswith('$$'):
                         is_special_op = True
         elif len(expectation) == 1:
-            opname, spec = next(iteritems(expectation))
+            opname, spec = next(iter(expectation.items()))
             if opname.startswith('$$'):
                 is_special_op = True
                 key_to_compare = None
@@ -523,7 +546,7 @@ class MatchEvaluatorUtil(object):
             return
 
         self.test.assertIsInstance(actual, abc.Mapping)
-        for key, value in iteritems(expectation):
+        for key, value in expectation.items():
             if self._evaluate_if_special_operation(expectation, actual, key):
                 continue
 
@@ -553,13 +576,13 @@ class MatchEvaluatorUtil(object):
                     self.match_result(e, a, in_recursive_call=True)
                 return
 
-        # account for flexible numerics in element-wise comparison and unicode
-        # vs str on Python 2.
-        if not (isinstance(expectation, integer_types) or
-                isinstance(expectation, float) or
-                (isinstance(expectation, unicode_type) and not PY3)):
+        # account for flexible numerics in element-wise comparison
+        if (isinstance(expectation, int) or
+                isinstance(expectation, float)):
+            self.test.assertEqual(expectation, actual)
+        else:
             self.test.assertIsInstance(actual, type(expectation))
-        self.test.assertEqual(expectation, actual)
+            self.test.assertEqual(expectation, actual)
 
     def assertHasServiceId(self, spec, actual):
         if 'hasServiceId' in spec:
@@ -570,7 +593,7 @@ class MatchEvaluatorUtil(object):
                 self.test.assertIsNone(actual.service_id)
 
     def match_event(self, event_type, expectation, actual):
-        name, spec = next(iteritems(expectation))
+        name, spec = next(iter(expectation.items()))
 
         # every command event has the commandName field
         if event_type == 'command':
@@ -606,8 +629,7 @@ class MatchEvaluatorUtil(object):
         elif name == 'poolCreatedEvent':
             self.test.assertIsInstance(actual, PoolCreatedEvent)
         elif name == 'poolReadyEvent':
-            # PyMongo 3.X does not support PoolReadyEvent.
-            assert False
+            self.test.assertIsInstance(actual, PoolReadyEvent)
         elif name == 'poolClearedEvent':
             self.test.assertIsInstance(actual, PoolClearedEvent)
             self.assertHasServiceId(spec, actual)
@@ -702,7 +724,6 @@ class UnifiedSpecTestMixinV1(IntegrationTest):
     def setUpClass(cls):
         # super call creates internal client cls.client
         super(UnifiedSpecTestMixinV1, cls).setUpClass()
-
         # process file-level runOnRequirements
         run_on_spec = cls.TEST_SPEC.get('runOnRequirements', [])
         if not cls.should_run_on(run_on_spec):
@@ -717,7 +738,6 @@ class UnifiedSpecTestMixinV1(IntegrationTest):
 
     def setUp(self):
         super(UnifiedSpecTestMixinV1, self).setUp()
-
         # process schemaVersion
         # note: we check major schema version during class generation
         # note: we do this here because we cannot run assertions in setUpClass
@@ -1058,6 +1078,46 @@ class UnifiedSpecTestMixinV1(IntegrationTest):
         pool = get_pool(client)
         self.assertEqual(spec['connections'], pool.active_sockets)
 
+    def _testOperation_loop(self, spec):
+        failure_key = spec.get('storeFailuresAsEntity')
+        error_key = spec.get('storeErrorsAsEntity')
+        successes_key = spec.get('storeSuccessesAsEntity')
+        iteration_key = spec.get('storeIterationsAsEntity')
+        iteration_limiter_key = spec.get('numIterations')
+        for i in [failure_key, error_key]:
+            if i:
+                self.entity_map[i] = []
+        for i in [successes_key, iteration_key]:
+            if i:
+                self.entity_map[i] = 0
+        i = 0
+        global IS_INTERRUPTED
+        while True:
+            if iteration_limiter_key and i >= iteration_limiter_key:
+                break
+            i += 1
+            if IS_INTERRUPTED:
+                break
+            try:
+                if iteration_key:
+                    self.entity_map._entities[iteration_key] += 1
+                for op in spec["operations"]:
+                    self.run_entity_operation(op)
+                    if successes_key:
+                        self.entity_map._entities[successes_key] += 1
+            except Exception as exc:
+                if isinstance(exc, AssertionError):
+                    key = failure_key or error_key
+                else:
+                    key = error_key or failure_key
+                if not key:
+                    raise
+                self.entity_map[key].append({
+                    "error": str(exc),
+                    "time": time.time(),
+                    "type": type(exc).__name__
+                })
+
     def run_special_operation(self, spec):
         opname = spec['name']
         method_name = '_testOperation_%s' % (opname,)
@@ -1070,11 +1130,11 @@ class UnifiedSpecTestMixinV1(IntegrationTest):
 
     def run_operations(self, spec):
         for op in spec:
-            target = op['object']
-            if target != 'testRunner':
-                self.run_entity_operation(op)
-            else:
+            if op['object'] == 'testRunner':
                 self.run_special_operation(op)
+            else:
+                self.run_entity_operation(op)
+
 
     def check_events(self, spec):
         for event_spec in spec:
@@ -1117,7 +1177,7 @@ class UnifiedSpecTestMixinV1(IntegrationTest):
                 self.assertListEqual(sorted_expected_documents,
                                      actual_documents)
 
-    def run_scenario(self, spec):
+    def run_scenario(self, spec, uri=None):
         # maybe skip test manually
         self.maybe_skip_test(spec)
 
@@ -1134,8 +1194,7 @@ class UnifiedSpecTestMixinV1(IntegrationTest):
         # process createEntities
         self.entity_map = EntityMapUtil(self)
         self.entity_map.create_entities_from_spec(
-            self.TEST_SPEC.get('createEntities', []))
-
+            self.TEST_SPEC.get('createEntities', []), uri=uri)
         # process initialData
         self.insert_initial_data(self.TEST_SPEC.get('initialData', []))
 
